import logging


DEFAULT_SERVER_PORT: int = 8000
DEFAULT_SERVER_LISTEN_ADDRESS: str = "localhost"
DEFAULT_LOGGING_LEVEL = logging.INFO

SERVER_PORT: int = DEFAULT_SERVER_PORT
SERVER_LISTEN_ADDRESS: str = DEFAULT_SERVER_LISTEN_ADDRESS
